module demo(in_clk, in_reset, in_enable, in_key, in_busy, out_data, out_start, out_data_dac1, out_data_dac2, out_led);

input in_clk;
input in_reset;
input in_enable;
input in_busy;
input in_key;

output out_start;
output out_led;
output [15:0] out_data;
output [15:0] out_data_dac1;
output [15:0] out_data_dac2;


reg start;
reg [15:0] data;
reg [15:0] pattern_1;
reg [15:0] pattern_2;
reg [3:0] counter_pattern;
reg key1;
reg key2;
reg busy1;
reg busy2;
reg [3:0] mode;
reg [3:0] counter_pac;
reg [15:0] timer_delay;
reg [7:0] counter;
reg led;

wire key;
wire busy;


assign out_start = start;
assign out_data = data;
assign out_data_dac1 = pattern_1;
assign out_data_dac2 = pattern_2;
assign key = key1 & key2;
assign busy = !busy1 & busy2;
assign out_led = led;

//key1
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				key1 <= 1'd1;
			end
		else
			begin
				if(in_enable == 1'd1) key1 <= in_key;
				else key1 <= 1'd1;
 			end
	end

	
//key2
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				key2 <= 1'd0;
			end
		else
			begin
				if(in_enable == 1'd1) key2 <= ~key1;
				else key2 <= 1'd0;
			end
	end
	
	
//busy1
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				busy1 <= 1'd0;
			end
		else
			begin
				if(in_enable == 1'd1) busy1 <= in_busy;
				else busy1 <= 1'd0;
			end
	end

//busy2
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				busy2 <= 1'd0;
			end
		else
			begin
				if(in_enable == 1'd1) busy2 <= busy1;
				else busy2 <= 1'd0;
			end
	end
//mode
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				mode <= 4'd0;
			end
		else
			begin
				if(in_enable == 1'd1)
					begin
						if(key == 1'd1 && mode < 4'd4) mode <= mode + 1;
						else if(mode == 4'd4) mode <= 4'd0;
					end
				else mode <= 4'd0;
			end
	end
//data
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				data <= 16'd0;
			end
		else
			begin
				if(in_enable == 1'd1)
					begin
						if(mode == 4'd0)
							begin
								if(counter_pac == 4'd0) data <= 16'h0104;
								else if(counter_pac == 4'd1) data <= 16'h0280;
								else if(counter_pac == 4'd2) data <= 16'h0303;
								else if(counter_pac == 4'd3) data <= 16'h0480;
							end
						else if(mode == 4'd1)
							begin
								if(counter_pac == 4'd0) data <= 16'h0480;
								else if(counter_pac == 4'd1) data <= 16'h0280;
								else if(counter_pac == 4'd2) data <= 16'h0302;
								else if(counter_pac == 4'd3) data <= 16'h0144;								
							end
						else if(mode == 4'd2)
							begin
								if(counter_pac == 4'd0) data <= 16'h0480;
								else if(counter_pac == 4'd1) data <= 16'h0280;
								else if(counter_pac == 4'd2) data <= 16'h0301;
								else if(counter_pac == 4'd3) data <= 16'h0184;								
							end
						else if(mode == 4'd3)
							begin
								if(counter_pac == 4'd0) data <= 16'h0400;
								else if(counter_pac == 4'd1) data <= 16'h0280;
								else if(counter_pac == 4'd2) data <= 16'h0300;
								else if(counter_pac == 4'd3) data <= 16'h01C4;								
							end							
					end
				else data <= 16'd0;
			end
	end
//counter_pac
always @(posedge in_clk)
		begin
			if(in_reset == 1'd0)
				begin
					counter_pac <= 4'd0;
				end
			else
				begin
					if(in_enable == 1'd1)
						begin
							if(key == 1'd1) counter_pac <= 4'd0;
							else if(busy == 1'd1 && counter_pac < 4'd4) counter_pac <= counter_pac + 1;
						end
					else counter_pac <= 4'd0;
				end
		end		
//timer_delay
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				timer_delay <= 16'd0;
			end
		else
			begin
				if(start == 1'd0 && timer_delay < 16'd3500) timer_delay <= timer_delay + 1;
				else if(start == 1'd1) timer_delay <= 16'd0;
			end
	end	
//start
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				start <= 1'd0;
			end
		else
			begin
				if(in_enable == 1'd1)
					begin
						if(timer_delay == 16'd3500 && counter_pac != 4'd4 && counter_pac != 4'd4) start <= 1'd1;
						else if(timer_delay == 16'd0 && in_busy == 1'd0)  start <= 1'd0;
					end
				else start <= 1'd0;
			end
	end

//counter_pattern
always @(posedge in_clk)
	begin
		if(in_reset == 1'h0) counter_pattern <= 4'h0;
		else
			begin
				if(in_enable == 1'h1)
					begin
						if(counter_pattern < 4'd4) counter_pattern <= counter_pattern + 1;
						else if(counter_pattern == 4'd4) counter_pattern <= 4'h0;
					end
				else counter_pattern <= 4'h0;
			end
	end
//counter
always @(posedge in_clk)
	begin
		if(in_reset == 1'h0) counter <= 8'h00;
		else
			begin
				if(in_enable == 1'h1)
					begin
						if(counter_pattern == 4'h3 && counter < 8'd100) counter <= counter + 1;
						else if(counter == 8'd100) counter <= 8'd0;
					end
				else counter <= 8'h00;
			end
	end	
//pattern_1(sin)
always @(posedge in_clk)
	begin
		if(in_reset == 1'h0) pattern_1 <= 16'd0;
		else
			begin
				if(in_enable == 1'h1)
					begin
						if(counter == 8'd0) pattern_1 <= 16'd34824;
						else if(counter == 8'd1) pattern_1 <= 16'd36873;
						else if(counter == 8'd2) pattern_1 <= 16'd38906;
						else if(counter == 8'd3) pattern_1 <= 16'd40915;
						else if(counter == 8'd4) pattern_1 <= 16'd42892;
						else if(counter == 8'd5) pattern_1 <= 16'd44829;
						else if(counter == 8'd6) pattern_1 <= 16'd46718;  
						else if(counter == 8'd7) pattern_1 <= 16'd48552;
						else if(counter == 8'd8) pattern_1 <= 16'd50324; 
						else if(counter == 8'd9) pattern_1 <= 16'd52026; 
						else if(counter == 8'd10) pattern_1 <= 16'd53653;
						else if(counter == 8'd11) pattern_1 <= 16'd55197;
						else if(counter == 8'd12) pattern_1 <= 16'd56653;
						else if(counter == 8'd13) pattern_1 <= 16'd58014;
						else if(counter == 8'd14) pattern_1 <=	16'd59276; 
						else if(counter == 8'd15) pattern_1 <= 16'd60433;
						else if(counter == 8'd16) pattern_1 <= 16'd61480;
						else if(counter == 8'd17) pattern_1 <= 16'd62415;
						else if(counter == 8'd18) pattern_1 <= 16'd63232;
						else if(counter == 8'd19) pattern_1 <= 16'd63930; 
						else if(counter == 8'd20) pattern_1 <= 16'd64504;
						else if(counter == 8'd21) pattern_1 <= 16'd64953; 
						else if(counter == 8'd22) pattern_1 <= 16'd65275; 
						else if(counter == 8'd23) pattern_1 <= 16'd65469;
						else if(counter == 8'd24) pattern_1 <= 16'd65534;
						else if(counter == 8'd25) pattern_1 <= 16'd65469; 
						else if(counter == 8'd26) pattern_1 <= 16'd65275;
						else if(counter == 8'd27) pattern_1 <= 16'd64953;  
						else if(counter == 8'd28) pattern_1 <= 16'd64504; 
						else if(counter == 8'd29) pattern_1 <= 16'd63930; 
						else if(counter == 8'd30) pattern_1 <= 16'd63232; 
						else if(counter == 8'd31) pattern_1 <= 16'd62415;
						else if(counter == 8'd32) pattern_1 <= 16'd61480; 
						else if(counter == 8'd33) pattern_1 <= 16'd60433;
						else if(counter == 8'd34) pattern_1 <= 16'd59276; 
						else if(counter == 8'd35) pattern_1 <= 16'd58014;
						else if(counter == 8'd36) pattern_1 <= 16'd56653;
						else if(counter == 8'd37) pattern_1 <= 16'd55197; 
						else if(counter == 8'd38) pattern_1 <= 16'd53653;
						else if(counter == 8'd39) pattern_1 <= 16'd52026; 
						else if(counter == 8'd40) pattern_1 <= 16'd50324; 
						else if(counter == 8'd41) pattern_1 <= 16'd48552;
						else if(counter == 8'd42) pattern_1 <= 16'd46718; 
						else if(counter == 8'd43) pattern_1 <= 16'd44829; 
						else if(counter == 8'd44) pattern_1 <= 16'd42892;
						else if(counter == 8'd45) pattern_1 <= 16'd40915;  
						else if(counter == 8'd46) pattern_1 <= 16'd38906; 
						else if(counter == 8'd47) pattern_1 <= 16'd36873;
						else if(counter == 8'd48) pattern_1 <= 16'd34824;  
						else if(counter == 8'd49) pattern_1 <= 16'd32767;
					   else if(counter == 8'd50) pattern_1 <= 16'd30710;
						else if(counter == 8'd51) pattern_1 <= 16'd28661; 
						else if(counter == 8'd52) pattern_1 <= 16'd26628;
						else if(counter == 8'd53) pattern_1 <= 16'd24619;
						else if(counter == 8'd54) pattern_1 <= 16'd22642;
						else if(counter == 8'd55) pattern_1 <= 16'd20705;
						else if(counter == 8'd56) pattern_1 <= 16'd18816; 
						else if(counter == 8'd57) pattern_1 <= 16'd16982; 
						else if(counter == 8'd58) pattern_1 <=	16'd15210; 
						else if(counter == 8'd59) pattern_1 <= 16'd13508; 
						else if(counter == 8'd60) pattern_1 <= 16'd11881; 
						else if(counter == 8'd61) pattern_1 <= 16'd10337; 
						else if(counter == 8'd62) pattern_1 <= 16'd8881; 
						else if(counter == 8'd63) pattern_1 <= 16'd7520;
						else if(counter == 8'd64) pattern_1 <= 16'd6258; 
						else if(counter == 8'd65) pattern_1 <= 16'd5101; 
						else if(counter == 8'd66) pattern_1 <= 16'd4054; 
						else if(counter == 8'd67) pattern_1 <= 16'd3119;
						else if(counter == 8'd68) pattern_1 <= 16'd2302;
						else if(counter == 8'd69) pattern_1 <= 16'd1604; 
						else if(counter == 8'd70) pattern_1 <= 16'd1030; 
						else if(counter == 8'd71) pattern_1 <= 16'd581; 
						else if(counter == 8'd72) pattern_1 <= 16'd259; 
						else if(counter == 8'd73) pattern_1 <= 16'd65; 
						else if(counter == 8'd74) pattern_1 <= 16'd0;
						else if(counter == 8'd75) pattern_1 <= 16'd65;
						else if(counter == 8'd76) pattern_1 <= 16'd259;
						else if(counter == 8'd77) pattern_1 <= 16'd581;
						else if(counter == 8'd78) pattern_1 <= 16'd1030;
						else if(counter == 8'd79) pattern_1 <= 16'd1604;
						else if(counter == 8'd80) pattern_1 <= 16'd2302;
						else if(counter == 8'd81) pattern_1 <= 16'd3119;
						else if(counter == 8'd82) pattern_1 <= 16'd4054;
						else if(counter == 8'd83) pattern_1 <= 16'd5101; 
						else if(counter == 8'd84) pattern_1 <= 16'd6258; 
						else if(counter == 8'd85) pattern_1 <= 16'd7520; 
						else if(counter == 8'd86) pattern_1 <= 16'd8881; 
						else if(counter == 8'd87) pattern_1 <= 16'd10337; 
						else if(counter == 8'd88) pattern_1 <= 16'd11881; 
						else if(counter == 8'd89) pattern_1 <= 16'd13508;
						else if(counter == 8'd90) pattern_1 <= 16'd15210; 
						else if(counter == 8'd91) pattern_1 <= 16'd16982;
						else if(counter == 8'd92) pattern_1 <= 16'd18816; 
						else if(counter == 8'd93) pattern_1 <= 16'd20705; 
						else if(counter == 8'd94) pattern_1 <= 16'd22642; 
						else if(counter == 8'd95) pattern_1 <= 16'd24619;
						else if(counter == 8'd96) pattern_1 <= 16'd26628; 
						else if(counter == 8'd97) pattern_1 <= 16'd28661;
						else if(counter == 8'd98) pattern_1 <= 16'd30710;
						else if(counter == 8'd99) pattern_1 <= 16'd32767;
						
					end
				else pattern_1 <= 16'd0;
			end
	end
	
//pattern_2(pila)
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				pattern_2 <= 16'd0;
			end
		else
			begin
				if(in_enable == 1'd1)
					begin
						if(counter == 8'd0) pattern_2 <= 16'd0; 
						else if(counter == 8'd1) pattern_2 <= 16'd655; 
						else if(counter == 8'd2) pattern_2 <= 16'd1311; 
						else if(counter == 8'd3) pattern_2 <= 16'd1966;  
						else if(counter == 8'd4) pattern_2 <= 16'd2621;  
						else if(counter == 8'd5) pattern_2 <= 16'd3277;  
						else if(counter == 8'd6) pattern_2 <= 16'd3932;  
						else if(counter == 8'd7) pattern_2 <= 16'd4587;  
						else if(counter == 8'd8) pattern_2 <= 16'd5243;  
						else if(counter == 8'd9) pattern_2 <= 16'd5898;  
						else if(counter == 8'd10) pattern_2 <= 16'd6553;  
						else if(counter == 8'd11) pattern_2 <= 16'd7209;  
						else if(counter == 8'd12) pattern_2 <= 16'd7864;  
						else if(counter == 8'd13) pattern_2 <= 16'd8520;  
						else if(counter == 8'd14) pattern_2 <= 16'd9175;  
						else if(counter == 8'd15) pattern_2 <= 16'd9830;  
						else if(counter == 8'd16) pattern_2 <= 16'd10490;  
						else if(counter == 8'd17) pattern_2 <= 16'd11140;  
						else if(counter == 8'd18) pattern_2 <= 16'd11800;  
						else if(counter == 8'd19) pattern_2 <= 16'd12450;  
						else if(counter == 8'd20) pattern_2 <= 16'd13110;  
						else if(counter == 8'd21) pattern_2 <= 16'd13760;  
						else if(counter == 8'd22) pattern_2 <= 16'd14420;  
						else if(counter == 8'd23) pattern_2 <= 16'd15070;  
						else if(counter == 8'd24) pattern_2 <= 16'd15730;  
						else if(counter == 8'd25) pattern_2 <= 16'd16380;  
						else if(counter == 8'd26) pattern_2 <= 16'd17040;  
						else if(counter == 8'd27) pattern_2 <= 16'd17690;  
						else if(counter == 8'd28) pattern_2 <= 16'd18350; 
						else if(counter == 8'd29) pattern_2 <= 16'd19010;  
						else if(counter == 8'd30) pattern_2 <= 16'd19660;  
						else if(counter == 8'd31) pattern_2 <= 16'd20320;  
						else if(counter == 8'd32) pattern_2 <= 16'd20970;  
						else if(counter == 8'd33) pattern_2 <= 16'd21630;  
						else if(counter == 8'd34) pattern_2 <= 16'd22280;  
						else if(counter == 8'd35) pattern_2 <= 16'd22940;  
						else if(counter == 8'd36) pattern_2 <= 16'd23590;  
						else if(counter == 8'd37) pattern_2 <= 16'd24250;  
						else if(counter == 8'd38) pattern_2 <= 16'd24900;  
						else if(counter == 8'd39) pattern_2 <= 16'd25560;  
						else if(counter == 8'd40) pattern_2 <= 16'd26210;  
						else if(counter == 8'd41) pattern_2 <= 16'd26870; 
						else if(counter == 8'd42) pattern_2 <= 16'd27520; 
						else if(counter == 8'd43) pattern_2 <= 16'd28180;  
						else if(counter == 8'd44) pattern_2 <= 16'd28840;  
						else if(counter == 8'd45) pattern_2 <= 16'd29490;  
						else if(counter == 8'd46) pattern_2 <= 16'd30150; 
						else if(counter == 8'd47) pattern_2 <= 16'd30800;  
						else if(counter == 8'd48) pattern_2 <= 16'd31460;  
						else if(counter == 8'd49) pattern_2 <= 16'd32110; 
						else if(counter == 8'd50) pattern_2 <= 16'd32770; 
						else if(counter == 8'd51) pattern_2 <= 16'd33420; 
						else if(counter == 8'd52) pattern_2 <= 16'd34080; 
						else if(counter == 8'd53) pattern_2 <= 16'd34730; 
						else if(counter == 8'd54) pattern_2 <= 16'd35390; 
						else if(counter == 8'd55) pattern_2 <= 16'd36040; 
						else if(counter == 8'd56) pattern_2 <= 16'd36700; 
						else if(counter == 8'd57) pattern_2 <= 16'd37350; 
						else if(counter == 8'd58) pattern_2 <= 16'd38010; 
						else if(counter == 8'd59) pattern_2 <= 16'd38670; 
						else if(counter == 8'd60) pattern_2 <= 16'd39320; 
						else if(counter == 8'd61) pattern_2 <= 16'd39980; 
						else if(counter == 8'd62) pattern_2 <= 16'd40630; 
						else if(counter == 8'd63) pattern_2 <= 16'd41290; 
						else if(counter == 8'd64) pattern_2 <= 16'd41940; 
						else if(counter == 8'd65) pattern_2 <= 16'd42600; 
						else if(counter == 8'd66) pattern_2 <= 16'd43250; 
						else if(counter == 8'd67) pattern_2 <= 16'd43910; 
						else if(counter == 8'd68) pattern_2 <= 16'd44560; 
						else if(counter == 8'd69) pattern_2 <= 16'd45220; 
						else if(counter == 8'd70) pattern_2 <= 16'd45870; 
						else if(counter == 8'd71) pattern_2 <= 16'd46350; 
						else if(counter == 8'd72) pattern_2 <= 16'd47190; 
						else if(counter == 8'd73) pattern_2 <= 16'd47840; 
						else if(counter == 8'd74) pattern_2 <= 16'd48500; 
						else if(counter == 8'd75) pattern_2 <= 16'd49150; 
						else if(counter == 8'd76) pattern_2 <= 16'd49810; 
						else if(counter == 8'd77) pattern_2 <= 16'd50460; 
						else if(counter == 8'd78) pattern_2 <= 16'd51120; 
						else if(counter == 8'd79) pattern_2 <= 16'd51770; 
						else if(counter == 8'd80) pattern_2 <= 16'd52430; 
						else if(counter == 8'd81) pattern_2 <= 16'd53080; 
						else if(counter == 8'd82) pattern_2 <= 16'd53740; 
						else if(counter == 8'd83) pattern_2 <= 16'd54390; 
						else if(counter == 8'd84) pattern_2 <= 16'd55050; 
						else if(counter == 8'd85) pattern_2 <= 16'd55700; 
						else if(counter == 8'd86) pattern_2 <= 16'd56360; 
						else if(counter == 8'd87) pattern_2 <= 16'd57020; 
						else if(counter == 8'd88) pattern_2 <= 16'd57670; 
						else if(counter == 8'd89) pattern_2 <= 16'd58380; 
						else if(counter == 8'd90) pattern_2 <= 16'd58980; 
						else if(counter == 8'd91) pattern_2 <= 16'd59640; 
						else if(counter == 8'd92) pattern_2 <= 16'd60290; 
						else if(counter == 8'd93) pattern_2 <= 16'd60950; 
						else if(counter == 8'd94) pattern_2 <= 16'd61600; 
						else if(counter == 8'd95) pattern_2 <= 16'd62260; 
						else if(counter == 8'd96) pattern_2 <= 16'd62910; 
						else if(counter == 8'd97) pattern_2 <= 16'd63570; 
						else if(counter == 8'd98) pattern_2 <= 16'd64220; 
						else if(counter == 8'd99) pattern_2 <= 16'd64880; 
				
					end
				else pattern_2 <= 16'd0;					
			end
	end	
//led
always @(posedge in_clk)
	begin
		if(in_reset == 1'd0)
			begin
				led <= 1'd0;
			end
		else
			begin
				if(in_enable == 1'd1 ) led <= 1'd1;
				else led <= 1'd0;
			end
			
	end

endmodule